@@ -1,15 +1,6 @@
-/* lmdif.f -- translated by f2c (version 20020621).
-   You must link the resulting object file with the libraries:
-	-lf2c -lm   (in that order)
-*/
-
 #include "cminpack.h"
 #include <math.h>
-#define real __cminpack_real__
-#define min(a,b) ((a) <= (b) ? (a) : (b))
-#define max(a,b) ((a) >= (b) ? (a) : (b))
-#define TRUE_ (1)
-#define FALSE_ (0)
+#include "cminpackP.h"
 
 __cminpack_attr__
 int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, real *x, 
@@ -254,7 +245,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
     if (iflag < 0) {
 	goto TERMINATE;
     }
-    fnorm = __cminpack_func__(enorm)(m, fvec);
+    fnorm = __cminpack_enorm__(m, fvec);
 
 /*     initialize levenberg-marquardt parameter and iteration counter. */
 
@@ -310,7 +301,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
             for (j = 0; j < n; ++j) {
                 wa3[j] = diag[j] * x[j];
             }
-            xnorm = __cminpack_func__(enorm)(n, wa3);
+            xnorm = __cminpack_enorm__(n, wa3);
             delta = factor * xnorm;
             if (delta == 0.) {
                 delta = factor;
@@ -391,7 +382,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
                 wa2[j] = x[j] + wa1[j];
                 wa3[j] = diag[j] * wa1[j];
             }
-            pnorm = __cminpack_func__(enorm)(n, wa3);
+            pnorm = __cminpack_enorm__(n, wa3);
 
 /*           on the first iteration, adjust the initial step bound. */
 
@@ -406,7 +397,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
             if (iflag < 0) {
                 goto TERMINATE;
             }
-            fnorm1 = __cminpack_func__(enorm)(m, wa4);
+            fnorm1 = __cminpack_enorm__(m, wa4);
 
 /*           compute the scaled actual reduction. */
 
@@ -428,7 +419,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
                     wa3[i] += fjac[i + j * ldfjac] * temp;
                 }
             }
-            temp1 = __cminpack_func__(enorm)(n, wa3) / fnorm;
+            temp1 = __cminpack_enorm__(n, wa3) / fnorm;
             temp2 = (sqrt(par) * pnorm) / fnorm;
             prered = temp1 * temp1 + temp2 * temp2 / p5;
             dirder = -(temp1 * temp1 + temp2 * temp2);
@@ -476,7 +467,7 @@ int __cminpack_func__(lmdif)(__cminpack_decl_fcn_mn__ void *p, int m, int n, rea
                 for (i = 0; i < m; ++i) {
                     fvec[i] = wa4[i];
                 }
-                xnorm = __cminpack_func__(enorm)(n, wa2);
+                xnorm = __cminpack_enorm__(n, wa2);
                 fnorm = fnorm1;
                 ++iter;
             }
